/******************************* Copyright notice *******************************
Copyright (C)2002 by Andrei Loskutov <loskutov@gmx.de>. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
********************************************************************************/

/**
 * Created on 02.12.2002
 *
 * To change this generated comment edit the template variable "filecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of file comments go to
 * Window>Preferences>Java>Code Generation.
 */
package adiag.gui;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.HashMap;

import javax.swing.Action;

/**
 * Class to administrate key input
 * @author Andy the Great
 */
public class KeyServer implements KeyListener {


    /** all known actions */
    private HashMap actions;
    
    /**  Ctrl - key was pressed */
    private boolean ctrl_key;
    /** Shift - key was pressed */
    private boolean shift_key;
        
    /**
     * Constructor for KeyServer.
     */
    public KeyServer(HashMap actions) {
        super();
        this.actions = actions;        
    }

    /**
     * One key was pressed...
     * @param e java.awt.event.KeyEvent
     * @see KeyListener#keyPressed(KeyEvent)
     */
    public void keyPressed(KeyEvent e) {
        int keyCode = e.getKeyCode();
        //char c = e.getKeyChar();
        //System.out.println("pressed: key code = " + keyCode + " (" + KeyEvent.getKeyText(keyCode) + ")");
        switch (keyCode) {
            case KeyEvent.VK_SHIFT :
                {
                    shift_key = true;
                    break;
                }
            case KeyEvent.VK_CONTROL :
                {
                    ctrl_key = true;
                    break;
                }
        }

    }

    /**
     * One key was released...
     * @param e java.awt.event.KeyEvent
     * @see KeyListener#keyReleased(KeyEvent)
     */
    public void keyReleased(KeyEvent e) {
        int keyCode = e.getKeyCode();
        char c = e.getKeyChar();
        int modifiers = e.getModifiers();
        
        //System.out.println(e.getKeyText(keyCode) + " " + e.getKeyModifiersText(modifiers));
        if ((modifiers & InputEvent.CTRL_MASK) != 0 || (modifiers & InputEvent.ALT_GRAPH_MASK) != 0) {
            String s = KeyEvent.getKeyText(keyCode);
            c = s.length() == 1 ? s.charAt(0) : c;
        }
        Action a = null;
        switch (c) {
            case 'a' :
            case 'A' :
                a = (Action) actions.get(Commands.COMMAND_ABOUT);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_ABOUT));
                }
                return;
            case 'c' :
            case 'C' :
                a = (Action) actions.get(Commands.COMMAND_COPY);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_COPY));
                }
                return;
            case 'd' :
            case 'D' :
                a = (Action) actions.get(Commands.COMMAND_COPY_DATA);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_COPY_DATA));
                }
                return;
            case 'e' :
            case 'E' :
                a = (Action) actions.get(Commands.COMMAND_COLLAPSE_ALL);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_COLLAPSE_ALL));
                }
                return;
            case 'i' :
            case 'I' :
                a = (Action) actions.get(Commands.COMMAND_EXPLORE_ALL_BRANCHES);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_EXPLORE_ALL_BRANCHES));
                }
                return;
            case 'l' :
            case 'L' :
                a = (Action) actions.get(Commands.COMMAND_CHANGE_LOGFILE);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_CHANGE_LOGFILE));
                }
                return;
            case 'n' :
            case 'N' :
                a = (Action) actions.get(Commands.COMMAND_NEW_WINDOW);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_NEW_WINDOW));
                }
                return;
            case 'p' :
            case 'P' :
                a = (Action) actions.get(Commands.COMMAND_SET_PROPS);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_SET_PROPS));
                }
                return;
            case 'r' :
            case 'R' :
                a = (Action) actions.get(Commands.COMMAND_REDO_LAYOUT);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_REDO_LAYOUT));
                }
                return;
            case 's' :
            case 'S' :
                a = (Action) actions.get(Commands.COMMAND_SAVE_AS_IMAGE);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_SAVE_AS_IMAGE));
                }
                return;
            case 't' :
            case 'T' :
                a = (Action) actions.get(Commands.COMMAND_SAVE_AS_TEXT);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_SAVE_AS_TEXT));
                }
                return;
            case 'v' :
            case 'V' :
                a = (Action) actions.get(Commands.COMMAND_CHANGE_FILE);
                if(a.isEnabled()){
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_CHANGE_FILE));
                }
                return;
            default :
                break;
        }

        //System.err.println("pressed: key code = " + keyCode + " (" + KeyEvent.getKeyText(keyCode) + ")");
        switch (keyCode) {
            case KeyEvent.VK_F1 :
                {
                    a = (Action) actions.get(Commands.COMMAND_HELP);
                    a.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_HELP));
                    break;
                }
            case KeyEvent.VK_SUBTRACT :
            case KeyEvent.VK_ADD :
                {
                    int diff = keyCode == KeyEvent.VK_ADD ? +1 : -1;//+0.15f : -0.15f;
                    if ((modifiers & InputEvent.CTRL_MASK) != 0) {
                        //menuCreator.zoom_slider.setValue(menuCreator.zoom_slider.getValue() + diff);
                        Action act = (Action) actions.get(Commands.COMMAND_ZOOM);
                        act.actionPerformed(new ActionEvent(this, -1, "" + diff));
                        //zoom(scale_x_axis + diff, scale_y_axis + diff);
                    }
                    break;
                }
            case KeyEvent.VK_SHIFT :
                {
                    shift_key = false;
                    break;
                }
            case KeyEvent.VK_CONTROL :
                {
                    ctrl_key = false;
                    break;
                }
            case KeyEvent.VK_END :
                {
                    Action act = (Action) actions.get(Commands.COMMAND_END);
                    act.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_END));
                    break;
                }
            case KeyEvent.VK_HOME :
                {
                    Action act = (Action) actions.get(Commands.COMMAND_HOME);
                    act.actionPerformed(new ActionEvent(this, -1, Commands.COMMAND_HOME));
                    break;
                }
        }
    }

    /**
     * Overriden, but not used
     * @param e java.awt.event.KeyEvent
     * @see KeyListener#keyTyped(KeyEvent)
     */
    public void keyTyped(KeyEvent e) {}


    public boolean isKeyPressed(int keyCode){
        switch (keyCode) {
            case KeyEvent.VK_SHIFT :
                {
                    return shift_key;
                }
            case KeyEvent.VK_CONTROL :
                {
                    return ctrl_key;
                }
        }        
        return false;
    }
}



